/*
 * Copyright 2013 Canonical Ltd.
 *
 * This file is part of ubuntu-calculator-app.
 *
 * ubuntu-calculator-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * ubuntu-calculator-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../engine.js" as CALC
import "../formula.js" as F
import "../dateutils.js" as DateUtils

Page {

    property var formula: new F.Formula()
    property var screenFormula: [{_text:'', _operation: '', _number:''}]

    function formulaPush(visual) {
        try{
        var last = formula.push(visual);
        } catch(exception){
            return;
        }
        if(last !== undefined) { // Just to check it is not blank
            //if the element to add is a number or an unary operator it is added to the current line
            //otherwise the element is a binary operator and it needs to be added to a new line
            if(last.type === CALC.T_NUMBER || last.type === CALC.T_UNARY_MINUS){
                screenFormula[screenFormula.length - 1]._number += last.value.toString();
            } else if (last.type === CALC.T_OPERATOR){

            }
            else {
                screenFormula.push({_text:'', _operation: last.value.toString(), _number: ''})
            }
            formulaView.currentOperatorsChanged();
        }

    }

    function changeSign(){
        formula.changeSign();
        formulaView.currentOperatorsChanged();
    }

    function calculate() {
        try{
            var result = formula.calculate();
        } catch(exception) {
            formulaPop();
            calculate();
        }
        formulaView.headerItem.state = "calculationCompleted";
        screenFormula.push({_text:'', _operation: '=', _number: result.toString()});
        formulaView.currentOperatorsChanged();
    }

    function formulaPop(){
        if(screenFormula.length <= 1){
            clear();
            return;
        }
        formula.pop();
        screenFormula.pop();
        formulaView.currentOperatorsChanged();
    }

    function clear(){
        formulaView.headerItem.state = "";
        screenFormula = [{_text:'', _operation: '', _number:''}];
        formula = new F.Formula();
        formulaView.currentOperatorsChanged();
    }

    function addFromMemory(numberToAdd) {
        try{
            var temp = new CALC.Scanner(numberToAdd, new CALC.Context());
        }catch(e){
            return;
        }
        if(temp.tokens.length === 1 && temp.tokens.last().type === CALC.T_NUMBER){
            formulaPush(numberToAdd);
        }
    }

    Rectangle {
        anchors.fill: parent

        gradient: Gradient {
            GradientStop { position: 0.00; color: "#323A5D" }
            GradientStop { position: 0.83; color: "#6A6AA1" }
            GradientStop { position: 1.00; color: "#6899D7" }
        }

        ListView {
            id: formulaView
            anchors.fill: parent
            verticalLayoutDirection: ListView.BottomToTop
            clip: true

            property bool __wasAtYBegining: false
            property int __initialContentY: 0
            property bool __toBeRefresh: false
            property int __displaceDist: contentY - __initialContentY

            signal currentOperatorsChanged()

            function addCurrentToMemory() {
                if (screenFormula[screenFormula.length-1]._operation === '=') {
                    var currentDate = new Date();
                    if (memory.get(0).mainLabel.length === 0)
                        memory.get(0).mainLabel = '';
                    memory.get(0).isLastItem = false
                    memory.get(0).dateText = i18n.tr("A few seconds ago...")
                    memory.get(0).timeStamp = currentDate.getTime()
                    memory.setProperty(0, "timeStamp", currentDate.getTime());
                    memory.insert(0,{'dbId': -1, 'dateText': "", 'operators': [{_text:'', _operation:'', _number:''}], 'isLastItem': true, 'mainLabel': ''});
                    positionViewAtBeginning();
                }
            }

            function showError() {
                animateError.start()
            }

            PropertyAnimation {id: animateError; target: formulaView; properties: "color"; from: "#FFA0A0"; to: "#FFFFFF"; duration: 100}

            remove: Transition{
                NumberAnimation {property: "opacity"; to: 0; duration: 200 }
            }

            displaced: Transition{
                SequentialAnimation{
                    NumberAnimation {property: "scale"; to: 0.9; duration: 50 }
                    NumberAnimation {property: "y"; duration: 300 }
                    NumberAnimation {property: "scale"; to: 1; duration: 50 }
                }
            }

            onCurrentOperatorsChanged: {
                memory.set(0, {"operators": screenFormula});
                positionViewAtBeginning();
            }

            model: Memory{
                id: memory
                Component.onCompleted: {
                    storage.getCalculations(function(calc){
                        memory.append(calc);
                    });
                }
            }

            header: CalcKeyboard {}

            delegate: Screen {
                id: screen
                width: formulaView.width
                ops: operators

                onUseAnswer: addFromMemory(answerToUse, formulaData)
                onMainLabelUpdated: {
                    memory.get(index).mainLabel = newText;
                }

                // Update date label
                onDateUpdate: {
                    // If is in database
                    if (memory.get(index).dbId !== -1)
                        memory.get(index).dateText = DateUtils.formatRelativeTime(i18n, storage.fromIdToDate(memory.get(index).dbId));
                    // If is in memory but not in database
                    else if (memory.get(index).timeStamp)
                        memory.get(index).dateText = DateUtils.formatRelativeTime(i18n, memory.get(index).timeStamp);
                }

                onLabelTextUpdated: {
                    if(index === 0){
                        screenFormula[idx]._text = newText;
                        return;
                    }
                    memory.get(index).operators.get(idx)._text = newText;
                }

                onRemoveItem: {
                    storage.removeCalculation(memory.get(index));
                    memory.remove(index)
                }
            }

            Scrollbar {
                flickableItem: formulaView
                align: Qt.AlignTrailing
            }

            Component.onCompleted: {
                currentOperatorsChanged();
            }

            Component.onDestruction: {
                //update the storage
                var calculations = []
                for(var i=1; i<memory.count; i++){
                    var operators = memory.get(i).operators;
                    var newop = [];
                    if(operators.count !== undefined){
                    for(var j=0; j< operators.count; j++){
                        newop.push(operators.get(j));
                    }
                    }else{
                        newop = operators
                    }
                    var newElement = {'dbId': memory.get(i).dbId,
                        'isLastItem': false,
                        'mainLabel': memory.get(i).mainLabel,
                        'operators': newop};
                    calculations.push({"calc": newElement, "date": memory.get(i).timeStamp})
                }
                storage.saveCalculations(calculations)
            }

            onMovementStarted: {
                __wasAtYBegining = atYEnd
                __initialContentY = contentY
            }
            onContentYChanged: {
                if (__wasAtYBegining) {
                    if (__displaceDist > units.gu(5)) {
                        __toBeRefresh = true;
                    }
                    else if (dragging) {
                        __toBeRefresh = false;
                    }
                }
            }
            onMovementEnded: {
                if (__toBeRefresh) {
                    formulaView.addCurrentToMemory();
                    clear();
                    __toBeRefresh = false
                }
            }
        }
    }
}
